#
# videodev.py
#
# Python Digital Audio Recorder
#
# Copyright (C) 2003 Daniel Thompson <d.thompson@gmx.net> 
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#

import fcntl, struct

# ioctl constants
VIDIOCGCAP       = -2143521279
VIDIOCGCHAN      = -1070565886
VIDIOCSCHAN      =  1076917763
VIDIOCGTUNER     = -1070303740
VIDIOCSTUNER     =  1077179909
VIDIOCGPICT      = -2146535930
VIDIOCSPICT      =  1074689543
VIDIOCCAPTURE    =  1074034184
VIDIOCGWIN       = -2145356279
VIDIOCSWIN       =  1075869194
VIDIOCGFBUF      = -2146142709
VIDIOCSFBUF      =  1075082764
VIDIOCKEY        = -2146666995
VIDIOCGFREQ      = -2147191282
VIDIOCSFREQ      =  1074034191
VIDIOCGAUDIO     = -2144831984
VIDIOCSAUDIO     =  1076393489
VIDIOCSYNC       =  1074034194
VIDIOCMCAPTURE   =  1074820627
VIDIOCGMBUF      = -2138540524
VIDIOCGUNIT      = -2146142699
VIDIOCGCAPTURE   = -2146142698
VIDIOCSCAPTURE   =  1075082775
VIDIOCSPLAYMODE  =  1074558488
VIDIOCSWRITEMODE =  1074034201
VIDIOCGPLAYINFO  = -2129103334
VIDIOCSMICROCODE =  1075344923
VIDIOCGVBIFMT    = -2145356260
VIDIOCSVBIFMT    =  1075869213

class VideoDev:
	def __init__(self, fname):
		self.dev = open(fname, 'r')

	def close(self):
		self.dev.close()

	def get_format(self, defn):
		format = ""
		for member in defn:
			format = format + member[0]
		return format

	def get_generic(self, op, defn):
		format = self.get_format(defn)
		raw = fcntl.ioctl(self.dev, op, struct.pack('%dx' % struct.calcsize(format)))
		list = struct.unpack(format, raw)

		data = {}
		i = 0
		for member in defn:
			name = member[1]

			if 2 == len(member):
				data[name] = list[i]
			elif member[2] == 'mask':
				data[name] = []
				for f, msk in member[3]:
					if 0 != (list[i] & msk):
						data[name].append(f)
			else:
				for f, enum in member[3]:
					if list[i] == enum:
						data[name] = f
						break
			i += 1

		return data

	def set_generic(self, op, defn, data):
		format = self.get_format(defn)

		list = []
		for member in defn:
			name  = member[1]

			if 2 == len(member):
				list.append(data[name])
			elif member[2] == 'mask':
				i = len(list)
				list.append(0)
				for f, msk in member[3]:
					if f in data[name]:
						list[i] = list[i] | msk
			else:
				for f, enum in member[3]:
					if f == data[name]:
						list.append(enum)
						break
		fcntl.ioctl(self.dev, op, struct.pack(format, *list))
		
		
	TUNER = (
		('i',	'tuner'),
		('32s',	'name'),
		('L',	'rangelow'),
		('L',	'rangehigh'),
		('i',	'flags', 'mask', (
			('VIDEO_TUNER_PAL',	    1),
			('VIDEO_TUNER_NTSC',	    2),
			('VIDEO_TUNER_SECAM',	    4),
			('VIDEO_TUNER_LOW',  	    8),	# Uses KHz not MHz
			('VIDEO_TUNER_NORM',	   16),	# Tuner can set norm
			('VIDEO_TUNER_STEREO_ON', 128),	# Tuner is seeing stereo
			('VIDEO_TUNER_RDS_ON',    256),	# Tuner is seeing an RDS datastream
			('VIDEO_TUNER_MBS_ON',	  512),	# Tuner is seeing an MBS datastream
		), ),
		('h',	'mode', 'enum', (
			('VIDEO_MODE_PAL',          0),
			('VIDEO_MODE_NTSC',         1),
			('VIDEO_MODE_SECAM',        2),
			('VIDEO_MODE_AUTO',         3),
		), ),
		('h',	'signal'),
	)

	def get_tuner(self):
		return self.get_generic(VIDIOCGTUNER, self.TUNER)

	AUDIO = (
		('i',	'audio'),
		('H',	'volume'),
		('H',	'bass' ),
		('i',	'flags', 'mask', (
			('VIDEO_AUDIO_MUTE',	 1),
			('VIDEO_AUDIO_MUTABLE',	 2),
			('VIDEO_AUDIO_VOLUME',	 4),
			('VIDEO_AUDIO_BASS',	 8),
			('VIDEO_AUDIO_TREBLE',	16),
		), ),
		('16s',	'name'),
		('h',	'mode', 'mask', (
			('VIDEO_SOUND_MONO',	 1),
			('VIDEO_SOUND_STEREO',	 2),
			('VIDEO_SOUND_LANG1',	 4),
			('VIDEO_SOUND_LANG2',	 8),
		), ),
		('h',	'balance'),
		('h',	'step'),
	)

	def get_audio(self):
		return self.get_generic(VIDIOCGAUDIO, self.AUDIO)

	def set_audio(self, data):
		return self.set_generic(VIDIOCSAUDIO, self.AUDIO, data)
	
	
	FREQ = (('L',	'freq'),)

	def get_freq(self):
		return self.get_generic(VIDIOCGFREQ, self.FREQ)['freq']
		
	def set_freq(self, freq):
		return self.set_generic(VIDIOCSFREQ, self.FREQ, { 'freq' : freq })
		
		

# TEST CODE

if __name__ == '__main__':
	import time

	vd = VideoDev('/dev/radio')

	print vd.get_tuner()

	audio = vd.get_audio()
	print audio
	audio['volume'] = 65535;
	if 'VIDEO_AUDIO_MUTE' in audio['flags']:
		audio['flags'].remove('VIDEO_AUDIO_MUTE')
	vd.set_audio(audio)

	print vd.get_freq()

	vd.set_freq(1627699)

	vd.close()

	
